@@ -6,7 +6,9 @@
 #  As of the Change Date specified in that file, in accordance with the Business Source
 #  License, use of this software will be governed by the Apache License, Version 2.0,
 #  included in the file licenses/APL.txt.
+
 from pathlib import Path
+from typing import Any
 
 import click
 import numpy as np
@@ -18,24 +20,31 @@ from synnax.cli.channel import (
     channel_name_table,
     maybe_select_channel,
 )
-from synnax.cli.select import select_simple
 from synnax.ingest.row import RowIngestionEngine
 from synnax.io import ChannelMeta, ReaderType, RowFileReader, IO_FACTORY, IOFactory
 from synnax.telem import DataType, Rate, TimeStamp
 from synnax.channel import Channel
-from synnax.cli.console.rich import RichConsole
 from synnax.cli.flow import Context, Flow
 from synnax.cli.io import prompt_file
-from synnax.cli.telem import prompt_data_type_select
+from synnax.cli.telem import select_data_type
+from synnax.cli import default
 
 # A shorthand grouping to represent all values in a set.
 GROUP_ALL = "__all__"
 
 
 @click.command()
-@click.argument("_path", type=click.Path(exists=True), required=False, default=None)
-def ingest(_path: str | None):
-    flow = Flow(Context(console=RichConsole()))
+@click.argument("path_", type=click.Path(exists=True), required=False, default=None)
+def ingest(path_: str | None):
+    return pure_ingest(path_)
+
+
+def pure_ingest(
+    path_: Path | str | None,
+    client: Synnax | None = None,
+    ctx: Context = default.context(),
+) -> None:
+    flow = Flow(ctx)
     flow.add("initialize_reader", initialize_reader)
     flow.add("connect_client", _connect_client)
     flow.add("ingest_all", ingest_all)
@@ -45,8 +54,8 @@ def ingest(_path: str | None):
     flow.add("validate_start_time", validate_start_time)
     flow.add("create_channels", create_channels)
     flow.add("ingest", run_ingestion)
-    path = None if _path is None else Path(_path)
-    flow.run(IngestionCLI(IO_FACTORY, path), "initialize_reader")
+    path = None if path_ is None else Path(path_)
+    flow.run(IngestionCLI(IO_FACTORY, path, client), "initialize_reader")
 
 
 class IngestionCLI:
@@ -59,11 +68,11 @@ class IngestionCLI:
     db_channels: list[Channel] | None
     start: TimeStamp | None = None
 
-    def __init__(self, factory: IOFactory, path: Path | None):
+    def __init__(self, factory: IOFactory, path: Path | None, client: Synnax | None):
         self.path = path
         self.factory = factory
         self.reader = None
-        self.client = None
+        self.client = client
         self.filtered_channels = None
         self.not_found = None
         self.db_channels = None
@@ -99,14 +108,15 @@ def initialize_reader(
 
 def _connect_client(ctx: Context, cli: IngestionCLI) -> str | None:
     """Prompts the user to connect to a Synnax client."""
-    cli.client = connect_client(ctx)
+    if cli.client is None:
+        cli.client = connect_client(ctx)
     return "ingest_all" if cli.client else None
 
 
 def ingest_all(ctx: Context, cli: IngestionCLI) -> str | None:
     """Prompts the user to ingest all channels."""
     assert cli.reader is not None
-    if ctx.console.confirm("Would you like to ingest all channels?", default=True):
+    if ctx.console.ask("Would you like to ingest all channels?", default=True):
         cli.filtered_channels = cli.reader.channels()
         return "validate_channels_exist"
     else:
@@ -125,6 +135,12 @@ def channels_to_ingest(ctx: Context, cli: IngestionCLI) -> str | None:
     cli.filtered_channels = [ch for ch in channels if ch.name in all_names]
     return "validate_channels_exist"
 
+def cannot_cast_error(ctx: Context, actual: Any, ch: Channel) -> None:
+    ctx.console.error(
+        f"""Unable to cast column data type {actual}
+for {ch.name} to channel data type {ch.data_type.np}"""
+    )
+
 
 def validate_data_types(ctx: Context, cli: IngestionCLI) -> str | None:
     """Does an optimistic check on the first sample of each channel.  This isn't error
@@ -137,11 +153,7 @@ def validate_data_types(ctx: Context, cli: IngestionCLI) -> str | None:
         samples_type = d_types[ch.name].np
         ch_type = ch.data_type.np
         if not np.can_cast(samples_type, ch_type):
-            ctx.console.error(
-                f"""Unable to cast file data type {samples_type} 
-                for channel {ch.name} to data type {ch_type}"""
-            )
-            return
+            return cannot_cast_error(ctx, samples_type, ch)
         elif samples_type != ch_type:
             ctx.console.warn(
                 f"""Channel {ch.name} has data type {ch_type} but the file data type is
@@ -195,7 +207,7 @@ def validate_channels_exist(ctx: Context, cli: IngestionCLI) -> str | None:
     if len(cli.not_found) > 0:
         ctx.console.info("The following channels were not found in the database:")
         channel_name_table(ctx, [ch.name for ch in cli.not_found])
-        if not ctx.console.confirm("Would you like to create them?"):
+        if not ctx.console.ask("Would you like to create them?", default=True):
             return None
         return "create_channels"
 
@@ -213,13 +225,12 @@ def validate_start_time(ctx: Context, cli: IngestionCLI) -> str | None:
         # If there is no index, it means all channels are rate based or we've already
         # written the index data. In either case, we need to prompt the user to enter
         # the start timestamp.
-        _start = ctx.console.ask_int(
+        _start = ctx.console.ask(
             """Please enter the start timestamp of the file as a
-                            nanosecond UTC integer. If you'd like a converter,
-                            use https://www.epochconverter.com/""",
+            nanosecond UTC integer. If you'd like a converter,
+            use https://www.epochconverter.com/""",
             default=TimeStamp.now(),
         )
-        assert _start is not None
         cli.start = TimeStamp(_start)
     else:
         idx = _idx[0]
@@ -229,9 +240,8 @@ def validate_start_time(ctx: Context, cli: IngestionCLI) -> str | None:
         cli.start = TimeStamp(first[idx.name].to_numpy()[0])
 
     ctx.console.info(f"Identified start timestamp for file as {cli.start}.")
-    if not ctx.console.confirm("Is this correct?"):
+    if not ctx.console.ask("Is this correct?", default=True):
         return None
-
     return "ingest"
 
 
@@ -255,7 +265,14 @@ def create_indexes(
     return [ch for ch in options if ch.name not in names]
 
 
-def assign_dt(
+DATA_TYPE_OPTIONS = [
+    "Guess data types from file",
+    "Assign the same data type to all channels (excluding indexes)",
+    "Group channels by data type",
+]
+
+
+def assign_data_type(
     ctx: Context,
     cli: IngestionCLI,
 ) -> dict[DataType, list[ChannelMeta]] | None:
@@ -263,17 +280,12 @@ def assign_dt(
 
     grouped = {GROUP_ALL: cli.db_channels}
     assigned = {}
-    opt = select_simple(
-        ctx,
-        [
-            "Guess data types from file",
-            "Assign the same data type to all channels (excluding indexes)",
-            "Group channels by data type",
-        ],
-        default=0,
-        required=True,
+    ctx.console.info("Please select an option for assigning data types:")
+    opt, _ = ctx.console.select(
+        rows=DATA_TYPE_OPTIONS,
+        default=DATA_TYPE_OPTIONS[0],
     )
-    if opt == 0:
+    if opt == DATA_TYPE_OPTIONS[0]:
         data_types = read_data_types(ctx, cli)
         assigned = {}
         for ch in cli.not_found:
@@ -283,7 +295,7 @@ def assign_dt(
             else:
                 assigned[dt].append(ch)
         return assigned
-    elif opt == 2:
+    elif opt == DATA_TYPE_OPTIONS[2]:
         groups = prompt_group_channel_names(ctx, [ch.name for ch in cli.not_found])
         if groups is None or len(groups) == 0:
             return None
@@ -293,12 +305,12 @@ def assign_dt(
     for key, group in grouped.items():
         if key != GROUP_ALL:
             ctx.console.info(f"Assigning data type to {key}")
-        dt = prompt_data_type_select(ctx)
+        dt = select_data_type(ctx)
         assigned[dt] = group
     return assigned
 
 
-def assign_idx(
+def assign_index_or_rate(
     ctx: Context,
     cli: IngestionCLI,
 ) -> dict[Rate | str, list[ChannelMeta]] | None:
@@ -309,10 +321,14 @@ def assign_idx(
     client = cli.client
 
     grouped = {GROUP_ALL: cli.not_found}
-    if not ctx.console.confirm(
-        "Do all non-indexed channels have the same data rate or index?"
+    if not ctx.console.ask(
+        "Do all non-indexed channels have the same data rate or index?",
+        bool,
+        default=True,
     ):
-        if not ctx.console.confirm("Can you group channels by data rate or index?"):
+        if not ctx.console.ask(
+            "Can you group channels by data rate or index?", default=True
+        ):
             grouped = {v.name: [v] for v in cli.not_found}
         grouped = prompt_group_channel_names(ctx, [ch.name for ch in cli.not_found])
         if grouped is None or len(grouped) == 0:
@@ -335,7 +351,7 @@ def assign_idx(
             idx = maybe_select_channel(ctx, res, _choice)
             if not idx:
                 ctx.console.warn(f"Index channel with key {_choice} not found")
-                if ctx.console.confirm("Try again?"):
+                if ctx.console.ask("Try again?", default=True):
                     return assign_to_group(key, group)
                 return None
             return idx.key
@@ -355,10 +371,10 @@ def create_channels(ctx: Context, cli: IngestionCLI) -> str | None:
     assert cli.client is not None
     assert cli.db_channels is not None
 
-    if ctx.console.confirm("Are any channels indexed (e.g. timestamps)?"):
+    if ctx.console.ask("Are any channels indexed (e.g. timestamps)?", default=True):
         cli.not_found = create_indexes(ctx, cli, cli.not_found)
 
-    idx_grouped, dt_grouped = assign_idx(ctx, cli), assign_dt(ctx, cli)
+    idx_grouped, dt_grouped = assign_index_or_rate(ctx, cli), assign_data_type(ctx, cli)
     if idx_grouped is None or dt_grouped is None:
         return None
 
